# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2025, by Samuel Williams.

require_relative "supervisor/version"

require_relative "supervisor/server"
require_relative "supervisor/worker"
require_relative "supervisor/client"

require_relative "supervisor/memory_monitor"

require_relative "supervisor/environment"
require_relative "supervisor/supervised"

# @namespace
module Async
	# @namespace
	module Container
		# @namespace
		module Supervisor
		end
	end
end
